/* Copyright (c) 2009 Nordic Semiconductor. All Rights Reserved.
 *
 * The information contained herein is property of Nordic Semiconductor ASA.
 * Terms and conditions of usage are described in detail in NORDIC
 * SEMICONDUCTOR STANDARD SOFTWARE LICENSE AGREEMENT.
 *
 * Licensees are granted free, non-transferable use of the information. NO
 * WARRANTY of ANY KIND is provided. This heading must NOT be removed from
 * the file.
 *
 */

/** @file
* 
* @defgroup gpiote_example_main main.c
* @{
* @ingroup nrf_gpiote_example
* @brief GPIOTE Example Application main file.
*
* This file contains the source code for a sample application using GPIOTE. 
*
* @image html example_board_setup_a.jpg "Use board setup A for this example."
*/

#include <stdbool.h>
#include <stdint.h>
#include "nrf.h"
#include "nrf_gpiote.h"
#include "nrf_gpio.h"
#include "boards.h"
#include "sound.h"


#define GPIO_OUTPUT_PIN_NUMBER   14     /**< Pin number for PWM output. */
#define GPIOTE_CHANNEL_NUMBER_0  0      /**< GPIOTE channel number. */
#define GPIOTE_CHANNEL_NUMBER_1  0      /**< GPIOTE channel number. */

#define MAX_LENGTH 256
#define PULSE_WIDTH 128




/** @brief Function for initializing the GPIO Tasks and Events peripheral.
*/
static void gpiote_init(void)
{
    // Configure GPIO_OUTPUT_PIN_NUMBER as an output.
    nrf_gpio_cfg_output(GPIO_OUTPUT_PIN_NUMBER);

    // Configure GPIOTE_CHANNEL_NUMBER to toggle the GPIO pin state with input.
    // @note Only one GPIOTE task can be coupled to an output pin.
    nrf_gpiote_task_config(GPIOTE_CHANNEL_NUMBER_0, GPIO_OUTPUT_PIN_NUMBER, \
                           NRF_GPIOTE_POLARITY_TOGGLE, NRF_GPIOTE_INITIAL_VALUE_LOW);

}

/** @brief Function for initializing Programmable Peripheral Interconnect (PPI) peripheral.
*   The PPI is needed to convert the timer event into a task.
 */
static void ppi_init(void)
{
    // Configure PPI channel 0 to toggle GPIO_OUTPUT_PIN on every TIMER0 COMPARE[0] match (200 ms)
    NRF_PPI->CH[0].EEP = (uint32_t)&NRF_TIMER2->EVENTS_COMPARE[0];
    NRF_PPI->CH[0].TEP = (uint32_t)&NRF_GPIOTE->TASKS_OUT[GPIOTE_CHANNEL_NUMBER_0];
    
    NRF_PPI->CH[1].EEP = (uint32_t)&NRF_TIMER2->EVENTS_COMPARE[1];
    NRF_PPI->CH[1].TEP = (uint32_t)&NRF_GPIOTE->TASKS_OUT[GPIOTE_CHANNEL_NUMBER_1];

    // Enable PPI channel 0
    NRF_PPI->CHEN = ((PPI_CHEN_CH0_Enabled << PPI_CHEN_CH0_Pos)
                    |(PPI_CHEN_CH1_Enabled << PPI_CHEN_CH1_Pos));
}


/** @brief Function for initializing the Timer 0 peripheral.
 */
static void timer2_init(void)
{
    // Start 16 MHz crystal oscillator.
    NRF_CLOCK->EVENTS_HFCLKSTARTED = 0;
    NRF_CLOCK->TASKS_HFCLKSTART    = 1;

    // Wait for the external oscillator to start.
    while (NRF_CLOCK->EVENTS_HFCLKSTARTED == 0) 
    {
        // Do nothing.
    }

    // Clear TIMER0
    NRF_TIMER2->TASKS_CLEAR = 1;

    // Configure TIMER0 for compare[0] event every 200 ms.
    NRF_TIMER2->PRESCALER = 1;       
    NRF_TIMER2->CC[0]     = 0;       
    NRF_TIMER2->CC[1]     = PULSE_WIDTH;
    NRF_TIMER2->CC[2]     = MAX_LENGTH;  
    NRF_TIMER2->MODE      = TIMER_MODE_MODE_Timer;
    NRF_TIMER2->BITMODE   = TIMER_BITMODE_BITMODE_08Bit;
    NRF_TIMER2->INTENSET  = (TIMER_INTENSET_COMPARE2_Enabled << TIMER_INTENSET_COMPARE2_Pos);
    
    NVIC_SetPriority(TIMER2_IRQn, 1); 
    NVIC_EnableIRQ(TIMER2_IRQn);
}

void load_sample(void)
{
    static uint32_t index;
    
    NRF_TIMER2->CC[1] = samples[index++];
    if(sample_length == index)
        {
            index = 0;
            NRF_TIMER2->TASKS_STOP=1;
        };
    
    
}    
/** @brief Function handling timer2 compare[2] event.
 */

void TIMER2_IRQHandler(void)
{
    load_sample();
    NRF_TIMER2->EVENTS_COMPARE[2] = 0;
    
}


/**
 * @brief Function for application main entry.
 */
int main(void)
{    
    gpiote_init();                // Configure a GPIO to toggle on a GPIOTE task.
    timer2_init();                // Use TIMER0 to generate events every 200 ms.
    ppi_init();                   // Use a PPI channel to connect the event to the task automatically.

    NRF_TIMER2->TASKS_START = 1;  // Start event generation.

    while (true)
    {
        __ASM("nop"); // replace with pwr mng
    }
}
/** @} */
